/*****************************************************
* 　リアルタイムクロック　ＤＳ１３０７
* 　　　2020/06/09
*   PIC16F1938　使用　　/　Ｉ２Ｃ　デバイス
* 
* 　I2Cライブラリーは
* 「電子工作のためのPIC16F活用ガイドブック」後閑哲也　著
*　 にて公開されているプログラムをPIC16F1938用に変更
* 　して使用しています。
******************************************************/

#include <xc.h>
#include <stdio.h>
#include  "i2c_lib4.h"

/*****コンフィグレーション設定*********/
// コンフィギュレーション１の設定
#pragma config FOSC = INTOSC    // 内部ｸﾛｯｸを使用する(INTOSC)
#pragma config WDTE = OFF       // ｳｵｯﾁﾄﾞｯｸﾞﾀｲﾏｰ無し(OFF)
#pragma config PWRTE = ON       // 電源ONから64ms後にﾌﾟﾛｸﾞﾗﾑを開始する(ON)
#pragma config MCLRE = ON       // 外部ﾘｾｯﾄ信号を使用
#pragma config CP = OFF         // ﾌﾟﾛｸﾞﾗﾑﾒﾓﾘｰを保護しない(OFF)
#pragma config CPD = OFF        // ﾃﾞｰﾀﾒﾓﾘｰを保護しない(OFF)
#pragma config BOREN = ON       // 電源電圧降下常時監視機能ON(ON)
#pragma config CLKOUTEN = OFF   // CLKOUTﾋﾟﾝをRA6ﾋﾟﾝで使用する(OFF)
#pragma config IESO = OFF       // 外部・内部ｸﾛｯｸの切替えでの起動はなし(OFF)
#pragma config FCMEN = OFF      // 外部ｸﾛｯｸ監視しない(FCMEN_OFF)

// コンフィギュレーション２の設定
#pragma config WRT = OFF        // Flashﾒﾓﾘｰを保護しない(OFF)
#pragma config PLLEN = OFF      // 動作クロックを32MHzでは動作させない(OFF)
#pragma config STVREN = ON      // スタックがオーバフローやアンダーフローしたらリセットをする(ON)
#pragma config BORV = HI        // 電源電圧降下常時監視電圧(2.5V)設定(HI)
#pragma config LVP = OFF        // 低電圧プログラミング機能使用しない(OFF)
#define  _XTAL_FREQ     8000000

void putch(unsigned char byte);							//シリアル通信用
void RTCWrite(unsigned int RTCAdrs, unsigned char Data);//RTCデータ書き込み用
unsigned char RTCRead(unsigned int RTCAdrs);			//RTCデータ読み込み用

/********　メイン関数　　************/
void main(void) {

	unsigned int rtcdata;   //ＲＴＣ　データ用
	
	/*******初期設定******/
    OSCCON = 0x72;          //内臓クロック８HMz指定(BPLL Off
    ANSELA = 0x00;          //A/D停止
    ANSELB = 0x00;          //A/D停止
    TRISA  = 0b00000000;    //
    TRISB  = 0b00000000;    //
    TRISC  = 0b10011000;    //SCLとSDAのポートは入力設定
	
    /***** IC2設定********/
    SSPSTAT = 0b10000000;   //　状態クリア
    SSPCON1 = 0b00101000;   // I2C　イネーブル
    SSPADD = 0x13;          // 100kHz@32MHz
 
    /****** UART設定 *******/
    TXSTA = 0x26;           //シリアル送信設定　0x24　でもok?
    RCSTA = 0x90;           //シリアル受信設定
    BAUDCON=0x08;           //
    SPBRG = 207;            //ボーレート設定 (9400bps)

    GIE = 0;                //割り込み無し

	/************　I２C　通信スタート　　********************/
	printf("\r\n HELLO ! Ver.1 \r\n");
    
    printf("RTC Write TEST ");
    
 	RTCWrite(0x00,0x06);    // 6秒をセットする
 	RTCWrite(0x01,0x24);    // 24分をセットする
    RTCWrite(0x02,0x03);    // 3時間をセットする
    RTCWrite(0x04,0x12);    // 12日をセットする
    RTCWrite(0x05,0x10);    // 10月をセットする
    RTCWrite(0x06,0x20);    // 2020年をセットする
	printf("/　OK! \r\n");
    
	printf("RTC Read TEST \r\n");
            
	while (1) {
        rtcdata = RTCRead(0x00);			//レジスタ00h(秒)を読み込む
        printf("Seconds %02X /",rtcdata);
        rtcdata = RTCRead(0x01);			//レジスタ01h(分)を読み込む
        printf("Minutes %02X /",rtcdata);
        rtcdata = RTCRead(0x02);			//レジスタ02h(時)を読み込む
        printf("Hours %02X /",rtcdata);
            
        rtcdata = RTCRead(0x04);			//レジスタ04h(日)を読み込む
        printf("Date %02X /",rtcdata);
        rtcdata = RTCRead(0x05);             //レジスタ05h(月)を読み込む
        printf("Month %02X /",rtcdata);
        rtcdata = RTCRead(0x06);			//レジスタ06h(年)を読み込む
         printf("Year 20%02X \r\n",rtcdata);            
                                    
        __delay_ms(5000);                    //

	}
}

/****************************************
 * USART 用
 **********************************/
void putch(unsigned char byte){
		
		RCSTAbits.SPEN=1;
		TXREG=byte;
		while(!TXIF)continue;
		TXIF=0;
}

/****************************************************************************
 * I2C デバイスとの通信
 ****************************************************************************/

/*************************************
* EEPROM 1バイト書き込み関数
 * チップアドレス　A0h
**************************************/
void EEWrite(unsigned int EEAdrs, unsigned char Data){
    unsigned int i;
    IdleI2C();							//アイドル待ち
    SSPCON2bits.SEN = 1;				//スタート出力
    while(SSPCON2bits.SEN);				//スタート終了待ち
    SendByte(0xA0);						//書き込みモード送信（チップアドレス）
    SendByte((EEAdrs >> 8) & 0xFF);		//書き込みアドレス上位送信
    SendByte(EEAdrs & 0xFF);			//書き込みアドレス下位送信
    SendByte(Data);						//書き込みデータ送信
    SSPCON2bits.PEN = 1;				//ストップ出力
    while(SSPCON2bits.PEN);				//ストップ終了待ち
	__delay_ms(3);						//書き込み完了待ち
										//2ms以上のウエイトが必要
}

/*************************************
* RTC 1バイト書き込み関数
* チップアドレス　D0h
**************************************/
void RTCWrite(unsigned int RTCAdrs, unsigned char Data){
		
    unsigned int i;
    IdleI2C();                             
    SSPCON2bits.SEN = 1;                   
    while(SSPCON2bits.SEN);                
    SendByte(0xD0);						//書き込みモード送信（チップアドレス）
    SendByte(RTCAdrs);						
    SendByte(Data);                        
    SSPCON2bits.PEN = 1;                   
    while(SSPCON2bits.PEN);                
	__delay_ms(3);                          
}

/*************************************
* RTC 1バイト読み込み関数
* チップアドレス　D0h
 *************************************/
unsigned char RTCRead(unsigned int RTCAdrs){
    unsigned char result;

    IdleI2C();
    SSPCON2bits.SEN = 1;
    while(SSPCON2bits.SEN);
    SendByte(0xD0);						//書き込みモード送信（チップアドレス）
    SendByte(RTCAdrs);
    /***** 読み出し *******/
    SSPCON2bits.ACKDT = 1;
    SSPCON2bits.SEN = 1;
    while(SSPCON2bits.SEN);
    SendByte(0xD1);						//読み込みモード送信（チップアドレス）
    SSPCON2bits.RCEN = 1;
    while(!SSPSTATbits.BF);
    SSPCON2bits.ACKEN = 1;
    IdleI2C();
    result = SSPBUF;
    SSPCON1bits.SSPOV = 0;
    SSPCON2bits.PEN = 1;
    while(SSPCON2bits.PEN);
    return(result);
}